/*
 * $Id: ArrayList.java,v 1.1 2007/03/26 06:54:40 mviara Exp $
 * $Name:  $
 *
 * Fast implementation of java.utill.ArrayList
 * 
 * $Log: ArrayList.java,v $
 * Revision 1.1  2007/03/26 06:54:40  mviara
 * Added support for interrupt statistics.
 *
 */
package j51.intel;

/**
 * Fast implementation without any syncronization and check of
 * of java.util.Arraylist. MCS51 make intensive usage of this class
 * and the performance are critical.
 */
public class ArrayList
{
	static final int GROW_SIZE = 10;
	Object objects[];
	int currentSize;

	ArrayList()
	{
		objects = new Object[0];
		currentSize = 0;
	}

	int size()
	{
		return currentSize;
	}

	private void checkSize()
	{
		if (currentSize >= objects.length)
		{
			Object newObjects[] = new Object[objects.length+GROW_SIZE];
			System.arraycopy(objects,0,newObjects,0,objects.length);
			objects = newObjects;
		}

	}

	void clear()
	{
		currentSize = 0;
	}

	void add(Object o)
	{
		checkSize();
		objects[currentSize++] = o;
	}

	void add(int i,Object o)
	{
		checkSize();
		System.arraycopy(objects,i,objects,i+1,currentSize - i);
		objects[i] = o;
		currentSize++;
	}

	int indexOf(Object o)
	{
		for (int i = 0 ; i < currentSize ; i++)
			if (objects[i] == o)
				return i;

		return -1;
	}

	void remove(int i)
	{
		if (i < currentSize && i >= 0)
		{
			objects[i] = null;

			int num = currentSize - i - 1;
			if (num > 0)
				System.arraycopy(objects,i+1,objects,i,num);
			currentSize--;
		}

	}


	void remove(Object o)
	{
		int i = indexOf(o);
		if (i != -1)
			remove(i);
	}

	Object get(int i)
	{
		return objects[i];
	}

	boolean contains(Object o)
	{
		return indexOf(o) != -1;
	}
}

