/**
 * $Id: C8051F120.java,v 1.2 2007/03/26 07:42:09 mviara Exp $
 * $Name:  $
 *
 * Silicon Laboratories C8051Fxxx.
 * 
 * Supported peripheral
 * 
 * - MOVX @Rx Hi byte of address in register EMIOCN
 * 
 * $Log: C8051F120.java,v $
 * Revision 1.2  2007/03/26 07:42:09  mviara
 * *** empty log message ***
 *
 * Revision 1.1  2007/03/24 09:13:14  mviara
 * Added preliminary support for Sil Labbs C8051Fxx
 *
 * 
 */
package j51.silabs;

import j51.intel.*;

interface C8051F120Constants
{
	public final int EMIOCN	= 0xa2;
	public final int PSBANK = 0xb1;
	
}

public class C8051F120 extends MCS51 implements Code,C8051F120Constants
{
	private PersistentBuffer	flash;
	private PersistentBuffer	scratch;
	
	C8051F120()
	{
		flash	= new PersistentBuffer("C8051F120","flash",128*1024);
		scratch = new PersistentBuffer("C8051F120","flash",256);
		
		setXdataSize(8192);
		setSfrXdataHi(EMIOCN);
		
		setSfrName(EMIOCN,	"EMIOCN");
		setSfrName(PSBANK,	"PSBANK");
	}

	public void reset()
	{
		super.reset();
		sfr(PSBANK,0x11);
	}

	int translate(int addr,boolean fetch)
	{

		if (addr > 0x8000)
		{
			int psBank = sfr(PSBANK);
			int bank;
			addr &= 0x7fff;
			if (fetch)
				bank = psBank & 0x03;
			else
				bank = (psBank >> 4 ) & 0x03;
			addr += bank * 0x8000;
		}

		return addr;
	}
	
	public void setCodeSize(int size)
	{
	}

	public int getCodeSize()
	{
		return 0x10000;
	}

	public void setCode(int addr,int value)
	{
		flash.set(addr,value);
	}

	public int getCode(int addr,boolean fetch)
	{
		return flash.get(translate(addr,fetch));
	}

	public int  getCode16(int addr,boolean fetch)
	{
		return ((getCode(addr,fetch) & 0xff) << 8 ) | (getCode(addr+1,fetch) & 0xff);
	}

	
}
